@@ -15,7 +15,18 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.            *
+ *                                                                         *
+ *   OpenSSL linking exception                                             *
+ *   --------------------------                                            *
+ *   If you modify this Program, or any covered work, by linking or        *
+ *   combining it with the OpenSSL project's "OpenSSL" library (or a       *
+ *   modified version of that library), containing parts covered by        *
+ *   the terms of OpenSSL/SSLeay license, the licensors of this            *
+ *   Program grant you additional permission to convey the resulting       *
+ *   work. Corresponding Source for a non-source form of such a            *
+ *   combination shall include the source code for the parts of the        *
+ *   OpenSSL library used as well as that of the covered work.             *
  ***************************************************************************/
 
 #include "fx_include.h"
@@ -152,9 +163,6 @@ void fx_bottom_on_setting_clicked(GtkWidget* UNUSED(widget) , gpointer data)
 	FxMain *fxmain = (FxMain*)data;
 	FxSet *fxset = fx_set_new(fxmain);
 	fx_set_initialize(fxset);
-
-	gtk_dialog_run(GTK_DIALOG(fxset->dialog));
-	gtk_widget_destroy(fxset->dialog);
 }
 
 void fx_bottom_on_sendtome_clicked(GtkWidget* UNUSED(widget) , gpointer data)
