@@ -15,10 +15,26 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.            *
+ *                                                                         *
+ *   OpenSSL linking exception                                             *
+ *   --------------------------                                            *
+ *   If you modify this Program, or any covered work, by linking or        *
+ *   combining it with the OpenSSL project's "OpenSSL" library (or a       *
+ *   modified version of that library), containing parts covered by        *
+ *   the terms of OpenSSL/SSLeay license, the licensors of this            *
+ *   Program grant you additional permission to convey the resulting       *
+ *   work. Corresponding Source for a non-source form of such a            *
+ *   combination shall include the source code for the parts of the        *
+ *   OpenSSL library used as well as that of the covered work.             *
  ***************************************************************************/
 
 #include "fx_include.h"
+#ifdef USE_INDICATE
+#include <libindicate/indicator.h>
+#include <libindicate/server.h>
+#include <libindicate/indicator-messages.h>
+#endif
 
 static GStaticMutex mutex;
 static void fx_chat_on_tophone_clicked(GtkWidget* widget , gpointer data);
@@ -34,7 +50,7 @@ FxChat* fx_chat_new(FxMain* fxmain , Conversation* conv)
 {
 	FxChat* fxchat = (FxChat*)malloc(sizeof(FxChat));
 
-	memset(fxchat , 0 , sizeof(FxChat));
+	memset(fxchat, 0, sizeof(FxChat));
 	fxchat->fxmain = fxmain;
 	fxchat->conv = conv;
 	fxchat->sendtophone = FALSE;
@@ -185,8 +201,33 @@ gboolean fx_chat_focus_in_func(GtkWidget *UNUSED(widget)
 	if(fxchat->unreadMsgCount > 0){
 		fxchat->unreadMsgCount = 0;
 		fx_chat_update_window(fxchat);
+#ifdef USE_INDICATE
+		GSList *iterator = NULL;
+		IndicateIndicator *indicator = NULL;
+		gint count = 0;
+		const gchar *id;
+		gchar *sid;
+		extern GSList *indicators;
+		sid = fetion_sip_get_sid_by_sipuri(fxchat->conv->currentContact->sipuri);
+
+		for (iterator = indicators; iterator; iterator = iterator->next, count ++) {
+			indicator = (IndicateIndicator*)iterator->data;
+			/* clear attention */
+			id = indicate_indicator_get_property(indicator, "sid");
+			if (strcmp(sid, id) == 0) {
+				indicate_indicator_set_property(indicator, INDICATE_INDICATOR_MESSAGES_PROP_ATTENTION, "");
+				/* remove the indicators from the indicate server */
+				indicate_server_remove_indicator(fxmain->indserver, indicator);
+				indicators = g_slist_remove(indicators, indicator);
+				break;
+			}
+		}
+		g_free(sid);
+#endif
 	}
 
+	if(!USE_STATUS_ICON(config)) return FALSE;
+
 	if(list_empty(ctlist)){
 		/* if the message queue is empty,and the status icon is not blinking
 		 * and there`s no unread mesasge for the current chat window,do nothing */
@@ -194,9 +235,6 @@ gboolean fx_chat_focus_in_func(GtkWidget *UNUSED(widget)
 				GTK_STATUS_ICON(fxmain->trayIcon))
 			 && fxchat->unreadMsgCount == 0)
 			return FALSE;
-
-		fx_head_set_state_image(fxmain,
-				fxmain->user->state);
 		gtk_status_icon_set_blinking(
 				GTK_STATUS_ICON(fxmain->trayIcon) , FALSE);
 		g_signal_handler_disconnect(fxmain->trayIcon,
@@ -207,11 +245,15 @@ gboolean fx_chat_focus_in_func(GtkWidget *UNUSED(widget)
 							 , "activate"
 							 , GTK_SIGNAL_FUNC(fx_main_tray_activate_func)
 							 , fxmain);
+
+		fx_head_set_state_image(fxmain,
+				fxmain->user->state);
+
 	}else{
 		message = (Message*)(ctlist->next->data);
-
 		sipuri = message->sipuri;
 		sid = fetion_sip_get_sid_by_sipuri(sipuri);
+
 		sprintf(path , "%s/%s.jpg" , config->iconPath , sid);
 		g_free(sid);
 
@@ -408,7 +450,7 @@ void fx_chat_initialize(FxChat* fxchat)
 		gtk_window_set_default_size(GTK_WINDOW(fxchat->dialog) , 600 , 430);
 	else
 		gtk_window_set_default_size(GTK_WINDOW(fxchat->dialog) , 550 , 430);
-	gtk_widget_set_size_request(fxchat->dialog , 550 , 0);
+	gtk_widget_set_size_request(fxchat->dialog, 550, 430);
 	gtk_container_set_border_width(GTK_CONTAINER(fxchat->dialog) , 10);
 	fx_chat_update_window(fxchat);
 
@@ -419,16 +461,15 @@ void fx_chat_initialize(FxChat* fxchat)
 	gtk_container_add(GTK_CONTAINER(fxchat->dialog) , vbox);
 	action_area = gtk_hbox_new(FALSE , 0);
 
-	fxchat->headbox = gtk_table_new(2 , 10 , FALSE );
+	fxchat->headbox = gtk_hbox_new(FALSE, 5);
 
 	pb = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"fetion.svg" , 40 , 40 , NULL);
 	gtk_window_set_icon(GTK_WINDOW(fxchat->dialog), pb);
 	fxchat->headimage = gtk_image_new_from_pixbuf(pb);
 	g_object_unref(pb);
 
-	gtk_table_attach(GTK_TABLE(fxchat->headbox) , fxchat->headimage
-								, 0 , 1 , 0 , 2
-								, GTK_FILL , GTK_FILL , 3 , 0);
+	gtk_box_pack_start(GTK_BOX(fxchat->headbox), fxchat->headimage,
+								FALSE, FALSE, 0);
 
 	fxchat->name_label = gtk_label_new(NULL);
 	sid = fetion_sip_get_sid_by_sipuri(contact->sipuri);
@@ -450,17 +491,22 @@ void fx_chat_initialize(FxChat* fxchat)
 
 	gtk_container_add(GTK_CONTAINER(fxchat->name_box) , fxchat->name_label);
 
-	gtk_table_attach(GTK_TABLE(fxchat->headbox) , fxchat->name_box
-								, 1 , 2 , 0 , 1
-								, GTK_FILL , GTK_FILL , 0 , 0);
+	GtkWidget *vbox1 = gtk_vbox_new(TRUE, 0);
+	GtkWidget *hbox1 = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(fxchat->headbox), vbox1,
+								FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox1), fxchat->name_box, FALSE, FALSE, 0);
+	fxchat->input_label = gtk_label_new(NULL);
+	gtk_box_pack_start(GTK_BOX(hbox1), fxchat->input_label, FALSE, FALSE, 10);
 
-	snprintf(nametext , 510 , "%s" , contact->impression);
+	snprintf(nametext , sizeof(nametext) - 1 , "%s" , contact->impression);
 	escape_impression(nametext);
 	fxchat->impre_label = gtk_label_new(nametext);
+	gtk_widget_set_size_request(fxchat->impre_label, 450, 20);
+	gtk_misc_set_alignment(GTK_MISC(fxchat->impre_label), 0, 0);
 	gtk_label_set_justify(GTK_LABEL(fxchat->impre_label) , GTK_JUSTIFY_LEFT);
-	gtk_table_attach(GTK_TABLE(fxchat->headbox) , fxchat->impre_label
-								, 1 , 10 , 1 , 2
-								, GTK_FILL , GTK_FILL , 0 , 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), fxchat->impre_label, FALSE, FALSE, 0);
 
 	halign = gtk_alignment_new( 0 , 0 , 0 , 0);
 	gtk_container_add(GTK_CONTAINER(halign) , fxchat->headbox);
@@ -1088,3 +1134,14 @@ void fx_chat_update_window(FxChat *fxchat)
 
 	gtk_window_set_title(GTK_WINDOW(fxchat->dialog), markup);
 }
+
+void fx_chat_set_input(FxChat *fxchat)
+{
+	gtk_label_set_markup(GTK_LABEL(fxchat->input_label), "正在输入...");
+}
+
+void fx_chat_clear_input(FxChat *fxchat)
+{
+	if(GTK_IS_LABEL(fxchat->input_label))
+		gtk_label_set_text(GTK_LABEL(fxchat->input_label), NULL);
+}
