@@ -15,8 +15,20 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.            *
+ *                                                                         *
+ *   OpenSSL linking exception                                             *
+ *   --------------------------                                            *
+ *   If you modify this Program, or any covered work, by linking or        *
+ *   combining it with the OpenSSL project's "OpenSSL" library (or a       *
+ *   modified version of that library), containing parts covered by        *
+ *   the terms of OpenSSL/SSLeay license, the licensors of this            *
+ *   Program grant you additional permission to convey the resulting       *
+ *   work. Corresponding Source for a non-source form of such a            *
+ *   combination shall include the source code for the parts of the        *
+ *   OpenSSL library used as well as that of the covered work.             *
  ***************************************************************************/
+
 #include <fx_include.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
@@ -33,6 +45,8 @@ int old_state;
 extern gint presence_count;
 extern struct userlist *ul;
 
+#define MAX_PASSWORD_LENGTH 16
+
 #ifdef USE_NETWORKMANAGER
 #include <dbus/dbus-glib.h>
 #include <NetworkManager.h>
@@ -41,7 +55,7 @@ static DBusGConnection *nm_conn = NULL;
 static DBusGProxy *nm_proxy = NULL;
 
 static void
-nm_state_change_cb(DBusGProxy *proxy, NMState state, gpointer data)
+nm_state_change_cb(DBusGProxy *UNUSED(proxy), NMState state, gpointer data)
 {
 	FxMain *fxmain = (FxMain*)data;
 	if(!fxmain->user)
@@ -245,6 +259,11 @@ int fx_conn_connect(FxMain *fxmain)
 	fetion_user_set_config(user , config);
 
 login:
+	newul = fetion_user_list_find_by_no(ul , no);
+	if(newul && strlen(password) == MAX_PASSWORD_LENGTH &&
+		strncmp(password, newul->password, MAX_PASSWORD_LENGTH) == 0)
+		fetion_user_set_password(user, newul->password);
+
 	pos = ssi_auth_action(user);
 	if(!pos){
 		fx_login_show_err(fxlogin , _("Login failed"));
@@ -302,34 +321,46 @@ login:
 	fx_main_history_init(fxmain);
 
 	/* set user list to be stored in local file	 */
+	newul = NULL;
 	newul = fetion_user_list_find_by_no(ul , no);
-	if(!newul){
-		if(remember)
-			newul = fetion_user_list_new(no,
-						  password , user->userId,
-						  user->sId, state , 1);
-		else
-			newul = fetion_user_list_new(no,
-						  NULL, user->userId,
-						  user->sId, state , 1);
-		foreach_userlist(ul , ul_cur)
-			ul_cur->islastuser = 0;
-		fetion_user_list_append(ul , newul);
-	}else{
-		memset(newul->password,
-				 0, sizeof(newul->password));
-		if(remember)
-			strcpy(newul->password , password);
+
+	if(!newul) {
+
+		if(remember) { 
+			gchar *hashed_password = hash_password(password);
+			newul = fetion_user_list_new(no, hashed_password, user->userId, user->sId, state, 1);
+			g_free(hashed_password);
+		} else newul = fetion_user_list_new(no,  NULL, user->userId,  user->sId, state, 1);
+
+		foreach_userlist(ul, ul_cur) ul_cur->islastuser = 0;
+		fetion_user_list_append(ul, newul);
+
+	} else {
+
+		if(remember) {
+			if(strlen(password) != MAX_PASSWORD_LENGTH ||
+				strncmp(password, newul->password, MAX_PASSWORD_LENGTH)) { /* hash password */
+				memset(newul->password, 0, sizeof(newul->password));
+				gchar *hashed_password = hash_password(password);
+				strcpy(newul->password, hashed_password);
+				g_free(hashed_password);
+			}
+		} else {
+			/* clear the password */
+			memset(newul->password, 0, sizeof(newul->password));
+		}
+
 		newul->laststate = state;
-		foreach_userlist(ul , ul_cur)
-			ul_cur->islastuser = 0;
+		foreach_userlist(ul , ul_cur) ul_cur->islastuser = 0;
 		newul->islastuser = 1;
 	}
+
 	fetion_user_list_save(config , ul);
 	fetion_user_list_free(ul);
 
 	/* download xml configuration file from the server */
 	fetion_config_load(user);
+
 	if(config->sipcProxyPort == 0)
 		fx_login_show_msg(fxlogin,
 					_("Detected that this is the first time you login\n"
@@ -342,6 +373,7 @@ login:
 					_("Connection has been shutdown by the server"));
 		return -1;
 	}
+
 	fetion_config_save(user);
 
 	fetion_user_set_st(user , state);
@@ -362,9 +394,19 @@ login:
 	}else{
 		fx_login_show_msg(fxlogin,
 					  _("Connecting to registration server"));
-		tcp_connection_connect(conn,
+		int ret = tcp_connection_connect(conn,
 					  	config->sipcProxyIP, 
 						config->sipcProxyPort);
+		if(ret == -1) {
+			config->sipcProxyPort = 443;
+			ret = tcp_connection_connect(conn,
+					  	config->sipcProxyIP, 
+						config->sipcProxyPort);
+			if(ret == -1) {
+				fx_login_show_err(fxlogin , _("Login failed"));
+				goto failed;
+			}
+		}
 	}
 
 	/* add the connection object into the connection list */
@@ -543,9 +585,12 @@ auth:
 			 user->nickname , user->mobileno);
 
 	gdk_threads_enter();
-	gtk_status_icon_set_tooltip(
-			GTK_STATUS_ICON(fxmain->trayIcon),
-		   	statusTooltip);
+
+	if(USE_STATUS_ICON(config)) {
+		gtk_status_icon_set_tooltip(
+				GTK_STATUS_ICON(fxmain->trayIcon),
+				statusTooltip);
+	}
 	/* set title of main window*/
 	gtk_window_set_title(GTK_WINDOW(fxmain->window),
 				   	user->nickname );
@@ -684,10 +729,17 @@ int fx_conn_reconnect(FxMain *fxmain, int state)
 					   	config->sipcProxyIP,
 					   	config->sipcProxyPort,
 					   	config->proxy);
-	else
+	else {
 		ret = tcp_connection_connect(tcp,
 					  	config->sipcProxyIP, 
 						config->sipcProxyPort);
+		if(ret == -1 && config->sipcProxyPort != 443) {
+			config->sipcProxyPort = 443;
+			ret = tcp_connection_connect(tcp,
+							config->sipcProxyIP, 
+							config->sipcProxyPort);
+		}
+	}
 
 	if(ret == -1){
 		gdk_threads_enter();
