@@ -15,7 +15,18 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.            *
+ *                                                                         *
+ *   OpenSSL linking exception                                             *
+ *   --------------------------                                            *
+ *   If you modify this Program, or any covered work, by linking or        *
+ *   combining it with the OpenSSL project's "OpenSSL" library (or a       *
+ *   modified version of that library), containing parts covered by        *
+ *   the terms of OpenSSL/SSLeay license, the licensors of this            *
+ *   Program grant you additional permission to convey the resulting       *
+ *   work. Corresponding Source for a non-source form of such a            *
+ *   combination shall include the source code for the parts of the        *
+ *   OpenSSL library used as well as that of the covered work.             *
  ***************************************************************************/
 
 #include "fx_include.h"
@@ -212,38 +223,39 @@ void fx_head_bind(FxMain* fxmain)
 }
 void fx_head_set_state_image(FxMain* fxmain , StateType type)
 {
-	FxHead* fxhead = fxmain->headPanel;
+	FxHead *fxhead = fxmain->headPanel;
 	GdkPixbuf *pixbuf;
-	char* statename = fx_util_get_state_name(type);
+	gchar *statename = fx_util_get_state_name(type);
+	gint use_status_icon = USE_STATUS_ICON(fxmain->user->config);
 
 	gtk_label_set_markup(GTK_LABEL(fxhead->state_label) , statename);
 	switch(type)
 	{
 		case P_ONLINE :
 			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"online.svg" , 20 , 20 , NULL);
-			gtk_status_icon_set_from_file(fxmain->trayIcon
-										, SKIN_DIR"online.svg");
+			if(use_status_icon)
+				gtk_status_icon_set_from_file(fxmain->trayIcon, SKIN_DIR"online.svg");
 			break;
 		case P_BUSY :
 			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"busy.svg" , 20 , 20 , NULL);
-			gtk_status_icon_set_from_file(fxmain->trayIcon
-										, SKIN_DIR"busy.svg");
+			if(use_status_icon)
+				gtk_status_icon_set_from_file(fxmain->trayIcon, SKIN_DIR"busy.svg");
 			break;
 		case P_HIDDEN :
 			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"invisible.svg" , 20 , 20 , NULL);
-			gtk_status_icon_set_from_file(fxmain->trayIcon
-										, SKIN_DIR"invisible.svg");
+			if(use_status_icon)
+				gtk_status_icon_set_from_file(fxmain->trayIcon, SKIN_DIR"invisible.svg");
 			break;
 		case P_OFFLINE :
 			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"offline.svg" , 20 , 20 , NULL);
-			gtk_status_icon_set_from_file(fxmain->trayIcon
-										, SKIN_DIR"offline.svg");
+			if(use_status_icon)
+				gtk_status_icon_set_from_file(fxmain->trayIcon, SKIN_DIR"offline.svg");
 			gtk_widget_set_sensitive(fxhead->portrait, FALSE);
 			break;
 		default :
 			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"away.svg" , 20 , 20 , NULL);
-			gtk_status_icon_set_from_file(fxmain->trayIcon
-										, SKIN_DIR"away.svg");
+			if(use_status_icon)
+				gtk_status_icon_set_from_file(fxmain->trayIcon, SKIN_DIR"away.svg");
 			break;
 	}
 	gtk_image_set_from_pixbuf(GTK_IMAGE(fxhead->state_img) , pixbuf);
@@ -585,7 +597,7 @@ void* fx_head_change_portrait_thread(void* data)
 	return NULL;
 }
 
-gboolean fx_head_set_sms_clicked(GtkWidget *widget , gpointer data)
+gboolean fx_head_set_sms_clicked(GtkWidget *UNUSED(widget), gpointer data)
 {
 	FxMain *fxmain = (FxMain*)data;
 	User *user = fxmain->user;
