@@ -15,7 +15,18 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.            *
+ *                                                                         *
+ *   OpenSSL linking exception                                             *
+ *   --------------------------                                            *
+ *   If you modify this Program, or any covered work, by linking or        *
+ *   combining it with the OpenSSL project's "OpenSSL" library (or a       *
+ *   modified version of that library), containing parts covered by        *
+ *   the terms of OpenSSL/SSLeay license, the licensors of this            *
+ *   Program grant you additional permission to convey the resulting       *
+ *   work. Corresponding Source for a non-source form of such a            *
+ *   combination shall include the source code for the parts of the        *
+ *   OpenSSL library used as well as that of the covered work.             *
  ***************************************************************************/
 
 #include "fx_include.h"
@@ -25,6 +36,8 @@ struct userlist  *ul;
 #define PORTRAIT_SIZE      120
 #define LOADING_IMAGE_SIZE 128
 
+#define MAX_PASSWORD_LENGTH 16
+
 static void userlist_remove_clicked(GtkWidget *widget, gpointer data);
 
 FxLogin* fx_login_new()
@@ -192,6 +205,7 @@ void fx_login_initialize(FxMain *fxmain)
 				   	GTK_JUSTIFY_CENTER);
 
 	fxlogin->password = gtk_entry_new();
+	gtk_entry_set_max_length(GTK_ENTRY(fxlogin->password), MAX_PASSWORD_LENGTH);
 	gtk_widget_set_size_request(GTK_WIDGET(fxlogin->password) , 240 , 25);
 	gtk_entry_set_visibility(GTK_ENTRY(fxlogin->password) , FALSE);
 	g_signal_connect(G_OBJECT(fxlogin->password)
@@ -435,6 +449,7 @@ void fx_login_set_last_login_user(FxLogin* fxlogin)
 	gchar         *pwd;
 	gint           state;
 	gint           last;
+	gchar           truncated[24];
 
 	if(!gtk_tree_model_get_iter_root(model , &iter))
 		return;
@@ -447,7 +462,16 @@ void fx_login_set_last_login_user(FxLogin* fxlogin)
 						 , -1);
 		if(last == 1){
 			gtk_combo_box_set_active_iter(combo , &iter);
-			gtk_entry_set_text(GTK_ENTRY(fxlogin->password) , pwd);
+
+			if(strlen(pwd) == 40) {/* this is a hashed password */
+				strncpy(truncated, pwd, MAX_PASSWORD_LENGTH);
+				pwd[MAX_PASSWORD_LENGTH] = '\0';
+			} else {
+				strncpy(truncated, pwd, strlen(pwd));
+				truncated[strlen(pwd)] = '\0';
+			}
+
+			gtk_entry_set_text(GTK_ENTRY(fxlogin->password), truncated);
 			fx_login_set_last_login_state(fxlogin , state);	
 			if(*pwd != '\0')
 				gtk_toggle_button_set_active(
@@ -489,11 +513,49 @@ void fx_login_user_change_func(GtkWidget* widget , gpointer data)
 	gchar          path[1024];
 	GtkTreeIter    iter;
 	gint           state;
-	struct userlist *ul_cur;
+	struct userlist *ul_cur = NULL;
+	const gchar    *nno;
+	gchar		   truncated[24];
+
+	GtkWidget *entry = gtk_bin_get_child(GTK_BIN(fxlogin->username));
+	nno = gtk_entry_get_text(GTK_ENTRY(entry));
+	if(!nno) return;
+	ul_cur = fetion_user_list_find_by_no(ul, nno);
+
+	if(!ul_cur) {
+		pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"fetion.svg",
+						120, 120, NULL);
+		if(pixbuf){
+			gtk_image_set_from_pixbuf(
+					GTK_IMAGE(fxlogin->portrait), pixbuf);
+			g_object_unref(pixbuf);
+		}
+		gtk_entry_set_text(GTK_ENTRY(fxlogin->password), "");
+	   	return;
+	}
 
 
-	if(!gtk_combo_box_get_active_iter(combo , &iter))
+	if(!gtk_combo_box_get_active_iter(combo , &iter)) {
+		config = fetion_config_new();
+		fetion_config_initialize(config, ul_cur->userid);
+		snprintf(path, sizeof(path) - 1, "%s/%s.jpg",
+				config->iconPath, ul_cur->sid);
+
+		pixbuf = gdk_pixbuf_new_from_file_at_size(path,
+						120, 120, NULL);
+		if(pixbuf){
+			gtk_image_set_from_pixbuf(
+					GTK_IMAGE(fxlogin->portrait), pixbuf);
+			g_object_unref(pixbuf);
+		}
+		gtk_entry_set_text(GTK_ENTRY(fxlogin->password), ul_cur->password);
+		fx_login_set_last_login_state(fxlogin , ul_cur->laststate);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxlogin->remember)
+								   , *(ul_cur->password) == '\0' ? FALSE : TRUE);
+
 		return;
+	}
+
 	gtk_tree_model_get(model, &iter,
 				 L_PWD_COL, &pwd,
 				 L_NO_COL, &no,
@@ -501,7 +563,7 @@ void fx_login_user_change_func(GtkWidget* widget , gpointer data)
 	ul_cur = fetion_user_list_find_by_no(ul, no);
 	config = fetion_config_new();
 	fetion_config_initialize(config, ul_cur->userid);
-	sprintf(path, "%s/%s.jpg",
+	snprintf(path, sizeof(path) - 1, "%s/%s.jpg",
 			config->iconPath, ul_cur->sid);
 
 	pixbuf = gdk_pixbuf_new_from_file_at_size(path,
@@ -511,7 +573,16 @@ void fx_login_user_change_func(GtkWidget* widget , gpointer data)
 				GTK_IMAGE(fxlogin->portrait), pixbuf);
 		g_object_unref(pixbuf);
 	}
-	gtk_entry_set_text(GTK_ENTRY(fxlogin->password) , pwd);
+
+	if(strlen(pwd) == 40) {/* this is a hashed password */
+		strncpy(truncated, pwd, MAX_PASSWORD_LENGTH);
+		pwd[MAX_PASSWORD_LENGTH] = '\0';
+	} else {
+		strncpy(truncated, pwd, strlen(pwd));
+		truncated[strlen(pwd)] = '\0';
+	}
+	gtk_entry_set_text(GTK_ENTRY(fxlogin->password), truncated);
+
 	fx_login_set_last_login_state(fxlogin , state);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxlogin->remember)
 							   , *pwd == '\0' ? FALSE : TRUE);
@@ -520,7 +591,7 @@ void fx_login_user_change_func(GtkWidget* widget , gpointer data)
 	g_free(no);
 }
 
-static void userlist_remove_clicked(GtkWidget *widget, gpointer data)
+static void userlist_remove_clicked(GtkWidget *UNUSED(widget), gpointer data)
 {
 	FxMain          *fxmain = (FxMain*)data;
 	FxLogin         *fxlogin = fxmain->loginPanel;
