@@ -15,7 +15,18 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.            *
+ *                                                                         *
+ *   OpenSSL linking exception                                             *
+ *   --------------------------                                            *
+ *   If you modify this Program, or any covered work, by linking or        *
+ *   combining it with the OpenSSL project's "OpenSSL" library (or a       *
+ *   modified version of that library), containing parts covered by        *
+ *   the terms of OpenSSL/SSLeay license, the licensors of this            *
+ *   Program grant you additional permission to convey the resulting       *
+ *   work. Corresponding Source for a non-source form of such a            *
+ *   combination shall include the source code for the parts of the        *
+ *   OpenSSL library used as well as that of the covered work.             *
  ***************************************************************************/
 
 #include "fx_include.h"
@@ -27,6 +38,12 @@
 #include <locale.h>
 #include <glib.h>
 #include <fx_server.h>
+#ifdef USE_INDICATE
+#include <libindicate/indicator.h>
+#include <libindicate-gtk/indicator.h>
+#include <libindicate/server.h>
+#include <libindicate/indicator-messages.h>
+#endif
 
 fd_set  fd_read;
 gint presence_count = 0;
@@ -38,11 +55,19 @@ gint start_popup_presence = 0;
 extern struct unacked_list *unackedlist;
 GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 GdkScreen *current_screen;
+FxMain *fxmain;
+#ifdef USE_INDICATE
+GSList *indicators = NULL;
+#endif
 
 static void fx_main_process_pggetgroupinfo(FxMain *fxmain , const gchar *sipmsg);
 static void fx_main_process_pgpresencechanged(FxMain *fxmain , const gchar *sipmsg);
 static gboolean key_press_func(GtkWidget *widget , GdkEventKey *event
 		, gpointer data);
+#ifdef USE_INDICATE
+static gint indicate_action();
+static void server_display(IndicateServer *server, guint timestamp);
+#endif // USE_INDICATE
 
 FxMain* fx_main_new()
 {
@@ -74,10 +99,11 @@ void fx_main_initialize(FxMain* fxmain)
 	gtk_widget_set_name(fxmain->window , "mainwindow");
 	gtk_window_set_title(GTK_WINDOW(fxmain->window) , "OpenFetion");
 
-
 	current_screen = gdk_screen_get_default();
 	config = fetion_config_new();
 	fetion_config_load_size(config);
+	fetion_config_get_use_status_icon(config);
+
 	if(config->window_width == 0){
 		int window_width , window_height;
 		window_width = gdk_screen_get_width(current_screen);
@@ -88,7 +114,6 @@ void fx_main_initialize(FxMain* fxmain)
 		window_pos_x = config->window_pos_x;
 		window_pos_y = config->window_pos_y;
 	}
-	fetion_config_free(config);
 
 	gtk_window_move(GTK_WINDOW(fxmain->window) , window_pos_x , window_pos_y);
 	gtk_container_set_border_width(GTK_CONTAINER(fxmain->window) , 0);
@@ -119,11 +144,8 @@ void fx_main_initialize(FxMain* fxmain)
 			GTK_WINDOW(fxmain->window),
 			WINDOW_WIDTH , WINDOW_HEIGHT);
 	GdkPixbuf* icon = gdk_pixbuf_new_from_file_at_size(
-			SKIN_DIR"fetion.svg" , 48 , 48 , NULL);
+			SKIN_DIR"fetion.svg" , 24 , 24 , NULL);
 	gtk_window_set_icon(GTK_WINDOW(fxmain->window) , icon);
-	fxmain->trayIcon = gtk_status_icon_new_from_file(
-			SKIN_DIR"offline.svg");
-	gtk_status_icon_set_tooltip(fxmain->trayIcon, "OpenFetion");
 #ifdef USE_LIBNOTIFY
 	#ifdef LIBNOTIFY_OLD
 		fxmain->notify = notify_notification_new("welcome", "", SKIN_DIR"offline.svg", NULL);
@@ -133,15 +155,20 @@ void fx_main_initialize(FxMain* fxmain)
 	notify_notification_set_timeout(fxmain->notify , 2500);
 #endif
 
-	fxmain->iconConnectId = g_signal_connect(
-						GTK_STATUS_ICON(fxmain->trayIcon),
-						"activate",
-						GTK_SIGNAL_FUNC(fx_main_tray_activate_func),
+	if(USE_STATUS_ICON(config)) { /* set status icon */
+		fxmain->trayIcon = gtk_status_icon_new_from_file(
+				SKIN_DIR"offline.svg");
+		gtk_status_icon_set_tooltip(fxmain->trayIcon, "OpenFetion");
+		fxmain->iconConnectId = g_signal_connect(
+							GTK_STATUS_ICON(fxmain->trayIcon),
+							"activate",
+							GTK_SIGNAL_FUNC(fx_main_tray_activate_func),
+							fxmain);
+		g_signal_connect(GTK_STATUS_ICON(fxmain->trayIcon),
+						"popup-menu",
+						GTK_SIGNAL_FUNC(fx_main_tray_popmenu_func),
 						fxmain);
-	g_signal_connect(GTK_STATUS_ICON(fxmain->trayIcon),
-				   	"popup-menu",
-				    GTK_SIGNAL_FUNC(fx_main_tray_popmenu_func),
-				    fxmain);
+	}
 
 	fxmain->mainbox = gtk_vbox_new(FALSE , 4);
 	gtk_container_add(GTK_CONTAINER(fxmain->window) , fxmain->mainbox);
@@ -150,6 +177,20 @@ void fx_main_initialize(FxMain* fxmain)
 	fx_login_initialize(fxmain);
 	gtk_widget_show_all(fxmain->window);
 
+	/* ubuntu indicator */
+	/* get the default indicator */
+#ifdef USE_INDICATE
+	IndicateServer *server = indicate_server_ref_default();
+	indicate_server_set_type(server, "message.openfetion");
+	indicate_server_set_desktop_file(server, DESKTOP_DIR"openfetion.desktop");
+	indicate_server_show(server);
+	fxmain->indserver = server;
+	g_signal_connect(G_OBJECT(server), INDICATE_SERVER_SIGNAL_SERVER_DISPLAY, G_CALLBACK(server_display), fxmain);
+#endif
+
+	/* free the temporary config object */
+	fetion_config_free(config);
+
 	gdk_threads_enter();
 	gtk_main();
 	gdk_threads_leave();
@@ -179,6 +220,7 @@ void update()
 		 gtk_main_iteration();
 	}
 }
+
 TimeOutArgs* timeout_args_new(FxMain *fxmain , FetionSip *sip , const gchar *sipuri)
 {
 	TimeOutArgs *args = (TimeOutArgs*)malloc(sizeof(TimeOutArgs));
@@ -189,6 +231,7 @@ TimeOutArgs* timeout_args_new(FxMain *fxmain , FetionSip *sip , const gchar *sip
 	strcpy(args->sipuri , sipuri);
 	return args;
 }
+
 GtkWidget* fx_main_create_menu(const gchar* name
 							 , const gchar* iconpath
 							 , GtkWidget* parent
@@ -208,7 +251,7 @@ GtkWidget* fx_main_create_menu(const gchar* name
 }
 
 GtkWidget* fx_main_create_menu1(const gchar* name
-							 , const gchar *stockid
+							 , const gchar *UNUSED(stockid)
 							 , GtkWidget* parent
 							 , void (*func)(GtkWidget* item , gpointer data)
 							 , gpointer data
@@ -316,15 +359,15 @@ static void popup_online_notify(FxMain *fxmain, Contact *contact)
 				config->iconPath , contact->sId);
 		sprintf(notifySummary,
 				_("%s , now ONLINE") , contact->nickname);
-		sprintf(notifyText ,
+		snprintf(notifyText, sizeof(notifyText) - 1,
 				_("Phone Number: %s\n"
 				  "Fetion Number: %s\n"
-				  "Signature: %s")
-				, contact->mobileno == NULL ||
-					strlen(contact->mobileno) == 0 ?
-					_("unknown") : contact->mobileno
-				, contact->sId
-				, contact->impression );
+				  "Signature: %s"),
+				contact->mobileno == NULL ||
+				strlen(contact->mobileno) == 0 ?
+				_("unknown") : contact->mobileno,
+				contact->sId,
+				contact->impression );
 		pixbuf = gdk_pixbuf_new_from_file_at_size(
 				iconPath,
 				NOTIFY_IMAGE_SIZE,
@@ -364,74 +407,78 @@ void fx_main_process_presence(FxMain* fxmain , const gchar* xml)
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeView));
 	foreach_contactlist(contactlist , contact){
 
-		if(fx_tree_get_buddy_iter_by_userid(model , contact->userId , &iter) == -1)
+		foreach_groupids(contact->groupids) {
+
+			if(fx_tree_get_buddy_iter(model, group_id, contact->userId, &iter) == -1)
 				continue;
 
-		presence_count ++;
+			presence_count ++;
 
-		/* all presence information has been pushed
-		 * then start update local data and buddy portrait */
-		if(presence_count == user->contactCount)
-			g_thread_create(update_data, fxmain, FALSE, NULL);
+			/* all presence information has been pushed
+			 * then start update local data and buddy portrait */
+			if(presence_count == user->contactCount)
+				g_thread_create(update_data, fxmain, FALSE, NULL);
 
-		gtk_tree_model_get(model , &iter
-						 , B_CRC_COL    , &crc
-						 , B_STATE_COL  , &oldstate
-						 , -1);
-		gdk_threads_enter();
-		fxchat = fx_list_find_chat_by_sipuri(fxmain->clist , contact->sipuri);
-		if(oldstate > 0 && contact->state <= 0 && contact->serviceStatus == STATUS_NORMAL)
-		{
-			gtk_tree_model_iter_parent(model , &parentIter , &iter);
-			gtk_tree_model_get(model , &parentIter
-							 , G_ONLINE_COUNT_COL	, &count
+			gtk_tree_model_get(model , &iter
+							 , B_CRC_COL    , &crc
+							 , B_STATE_COL  , &oldstate
 							 , -1);
-			count --;
-			gtk_tree_store_set(GTK_TREE_STORE(model) , &parentIter
-							 , G_ONLINE_COUNT_COL	, count
-							 , -1);
-			fx_tree_move_to_the_last(model , &iter);
-		}
-		if(oldstate <= 0 && contact->state > 0 && contact->serviceStatus == STATUS_NORMAL)
-		{
-			gtk_tree_model_iter_parent(model , &parentIter , &iter);
-			gtk_tree_model_get(model , &parentIter
-							 , G_ONLINE_COUNT_COL	, &count
-							 , -1);
-			count ++;
-			gtk_tree_store_set(GTK_TREE_STORE(model) , &parentIter
-							 , G_ONLINE_COUNT_COL	, count
-							 , -1);
-			fx_tree_move_to_the_last(model , &iter);
-			fx_tree_move_to_the_first(model , &iter);
+			gdk_threads_enter();
+			fxchat = fx_list_find_chat_by_sipuri(fxmain->clist , contact->sipuri);
+			if(oldstate > 0 && contact->state <= 0 && contact->serviceStatus == STATUS_NORMAL)
+			{
+				gtk_tree_model_iter_parent(model , &parentIter , &iter);
+				gtk_tree_model_get(model , &parentIter
+								 , G_ONLINE_COUNT_COL	, &count
+								 , -1);
+				count --;
+				gtk_tree_store_set(GTK_TREE_STORE(model) , &parentIter
+								 , G_ONLINE_COUNT_COL	, count
+								 , -1);
+				fx_tree_move_to_the_last(model , &iter);
+			}
+			if(oldstate <= 0 && contact->state > 0 && contact->serviceStatus == STATUS_NORMAL)
+			{
+				gtk_tree_model_iter_parent(model , &parentIter , &iter);
+				gtk_tree_model_get(model , &parentIter
+								 , G_ONLINE_COUNT_COL	, &count
+								 , -1);
+				count ++;
+				gtk_tree_store_set(GTK_TREE_STORE(model) , &parentIter
+								 , G_ONLINE_COUNT_COL	, count
+								 , -1);
+				fx_tree_move_to_the_last(model , &iter);
+				fx_tree_move_to_the_first(model , &iter);
 
-			popup_online_notify(fxmain, contact);
+				popup_online_notify(fxmain, contact);
 
-		}
-		if(fxchat)
-			fxchat->state = contact->state;
+			}
+			if(fxchat)
+				fxchat->state = contact->state;
 
-		name = (contact->nickname == NULL || strlen(contact->localname) == 0)?
-				contact->nickname : contact->localname;
-		gtk_tree_store_set(GTK_TREE_STORE(model) , &iter
-						 , B_NAME_COL            , g_markup_escape_text(name, -1)
-						 , B_SIPURI_COL			 , contact->sipuri
-						 , B_IMPRESSION_COL		 , g_markup_escape_text(contact->impression, -1)
-						 , B_PHONENUM_COL		 , contact->mobileno
-						 , B_USERID_COL			 , contact->userId
-						 , B_STATE_COL			 , contact->state
-						 , B_IDENTITY_COL		 , contact->identity
-						 , B_DEVICE_COL			 , contact->devicetype
-						 , B_RELATIONSTATUS_COL  , contact->relationStatus
-						 , B_SERVICESTATUS_COL	 , contact->serviceStatus
-						 , B_CARRIERSTATUS_COL   , contact->carrierStatus
-						 , B_CARRIER_COL		 , contact->carrier
-						 , B_CRC_COL			 , contact->portraitCrc
-						 , B_IMAGE_CHANGED_COL	 , crc == NULL ? IMAGE_CHANGED :
-						 (strcmp(crc , contact->portraitCrc) == 0 ? IMAGE_NOT_CHANGED : IMAGE_CHANGED)
-						 , -1);
-		g_free(crc);
-		gdk_threads_leave();
+			name = (contact->nickname == NULL || strlen(contact->localname) == 0)?
+					contact->nickname : contact->localname;
+			gtk_tree_store_set(GTK_TREE_STORE(model) , &iter
+							 , B_NAME_COL            , g_markup_escape_text(name, -1)
+							 , B_SIPURI_COL			 , contact->sipuri
+							 , B_IMPRESSION_COL		 , g_markup_escape_text(contact->impression, -1)
+							 , B_PHONENUM_COL		 , contact->mobileno
+							 , B_USERID_COL			 , contact->userId
+							 , B_STATE_COL			 , contact->state
+							 , B_IDENTITY_COL		 , contact->identity
+							 , B_DEVICE_COL			 , contact->devicetype
+							 , B_RELATIONSTATUS_COL  , contact->relationStatus
+							 , B_SERVICESTATUS_COL	 , contact->serviceStatus
+							 , B_CARRIERSTATUS_COL   , contact->carrierStatus
+							 , B_CARRIER_COL		 , contact->carrier
+							 , B_CRC_COL			 , contact->portraitCrc
+							 , B_IMAGE_CHANGED_COL	 , crc == NULL ? IMAGE_CHANGED :
+							 (strcmp(crc , contact->portraitCrc) == 0 ? IMAGE_NOT_CHANGED : IMAGE_CHANGED)
+							 , -1);
+			g_free(crc);
+			gdk_threads_leave();
+
+		} end_groupids(contact->groupids)
 	}
 }
 
@@ -451,9 +498,7 @@ static void process_system_message(const char *sipmsg)
 		sysmsg = fx_sysmsg_new();
 		fx_sysmsg_initialize(sysmsg);
 		fx_sysmsg_bind(sysmsg , msg , url);
-		gtk_dialog_run(GTK_DIALOG(sysmsg->dialog));
 
-		gtk_widget_destroy(sysmsg->dialog);
 		g_free(sysmsg);
 
 	}
@@ -462,7 +507,7 @@ static void process_system_message(const char *sipmsg)
 static void process_group_message(FxMain *fxmain , Message *message)
 {
 	FxPGGroup     *fxpgcur;
-	FxPGGroup     *fxpg;
+	FxPGGroup     *fxpg = NULL;
 	PGGroupMember *memcur;
 	FxList        *pglist;
 	FxList        *cur;
@@ -507,20 +552,23 @@ static void process_group_message(FxMain *fxmain , Message *message)
 			    pg_add_message(fxpg , message->message , &(message->sendtime) , memcur);
 		    }
 		}
-		sid = fetion_sip_get_pgid_by_sipuri(message->pguri);
-		snprintf(path , sizeof(path) - 1 , "%s/PG%s.jpg" , config->iconPath , sid);
-		g_free(sid);
-		pixbuf = gdk_pixbuf_new_from_file(path , NULL);
-		if(pixbuf == NULL)
-			pixbuf = gdk_pixbuf_new_from_file(SKIN_DIR"online.svg" , NULL);
-		gtk_status_icon_set_from_pixbuf(GTK_STATUS_ICON(fxmain->trayIcon) , pixbuf);
-		g_object_unref(pixbuf);
-		gtk_status_icon_set_blinking(GTK_STATUS_ICON(fxmain->trayIcon) , TRUE);
-		g_signal_handler_disconnect(fxmain->trayIcon , fxmain->iconConnectId);
-		fxmain->iconConnectId = g_signal_connect(G_OBJECT(fxmain->trayIcon)
-							, "activate"
-							, GTK_SIGNAL_FUNC(fx_main_message_func)
-							, fxmain);
+		
+		if(USE_STATUS_ICON(config)) { /* set the action of the status icon */
+			sid = fetion_sip_get_pgid_by_sipuri(message->pguri);
+			snprintf(path , sizeof(path) - 1 , "%s/PG%s.jpg" , config->iconPath , sid);
+			g_free(sid);
+			pixbuf = gdk_pixbuf_new_from_file(path , NULL);
+			if(pixbuf == NULL)
+				pixbuf = gdk_pixbuf_new_from_file(SKIN_DIR"online.svg" , NULL);
+			gtk_status_icon_set_from_pixbuf(GTK_STATUS_ICON(fxmain->trayIcon) , pixbuf);
+			g_object_unref(pixbuf);
+			gtk_status_icon_set_blinking(GTK_STATUS_ICON(fxmain->trayIcon) , TRUE);
+			g_signal_handler_disconnect(fxmain->trayIcon , fxmain->iconConnectId);
+			fxmain->iconConnectId = g_signal_connect(G_OBJECT(fxmain->trayIcon)
+								, "activate"
+								, GTK_SIGNAL_FUNC(fx_main_message_func)
+								, fxmain);
+		}
 	}else{
 		foreach_pg_member(fxpg->pggroup->member , memcur){
 		    if(strcmp(memcur->sipuri , message->sipuri) == 0)
@@ -568,6 +616,119 @@ static void popup_msg_notify(FxMain *fxmain, Contact *senderContact, Message *ms
 #endif
 }
 
+#ifdef USE_INDICATE
+static gint indicate_action() 
+{
+	GSList *iterator = NULL;
+	IndicateIndicator *indicator = NULL;
+	gint count = 0;
+
+
+	for (iterator = indicators; iterator; iterator = iterator->next, count ++) {
+		indicator = (IndicateIndicator*)iterator->data;
+		/* clear attention */
+		indicate_indicator_set_property(indicator, INDICATE_INDICATOR_MESSAGES_PROP_ATTENTION, "");
+		/* remove the indicators from the indicate server */
+		indicate_server_remove_indicator(fxmain->indserver, indicator);
+	}
+
+	/* free the memory allocated for the indicator list */
+	g_slist_free(indicators);
+	indicators = NULL;
+
+	if(count == 0) return 0;
+
+	/* process the incoming message */
+	fx_main_message_func(NULL, fxmain);
+
+	return count;
+}
+
+#ifdef USE_INDICATE
+static void server_display(IndicateServer *UNUSED(server), guint UNUSED(timestamp))
+{
+	if(indicate_action() == 0) {
+		//gtk_widget_show(fxmain->window);
+		if(window_pos_x_old == 0 && window_pos_y_old == 0){
+			window_pos_x_old = window_pos_x;
+			window_pos_y_old = window_pos_y;
+		}
+		gtk_window_move(GTK_WINDOW(fxmain->window),
+				window_pos_x_old , window_pos_y_old);
+		gtk_window_present(GTK_WINDOW(fxmain->window));
+	}
+}
+#endif //USE_INDICATE
+
+static void message_display(IndicateIndicator *UNUSED(indicator), gpointer UNUSED(data))
+{
+	indicate_action();
+}
+
+static IndicateIndicator *get_indicator(IndicateServer *server, const gchar *sid)
+{
+	GSList *iterator = NULL;
+	IndicateIndicator *indicator;
+	const gchar *id;
+	gchar portrait[1024];
+	GdkPixbuf *pixbuf;
+
+	for (iterator = indicators; iterator; iterator = iterator->next) {
+
+		indicator = (IndicateIndicator*)iterator->data;
+		id = indicate_indicator_get_property(indicator, "sid");
+
+		if(strcmp(sid, id) == 0) return indicator;
+	}
+
+	/* no indicator found, create one :) */
+	indicator = indicate_indicator_new();
+	/* add it to the global indicator list */
+	indicators = g_slist_append(indicators, indicator);
+	indicate_server_add_indicator(server, indicator);
+
+	indicate_indicator_set_property(indicator, INDICATE_INDICATOR_MESSAGES_PROP_COUNT, "0");
+	indicate_indicator_set_property(indicator, "sid", sid);
+	/* set icon */
+	snprintf(portrait, sizeof(portrait) - 1, "%s/%s.jpg", fxmain->user->config->iconPath, sid);
+	pixbuf = gdk_pixbuf_new_from_file(portrait, NULL);
+	indicate_indicator_set_property_icon(indicator, INDICATE_INDICATOR_MESSAGES_PROP_ICON, pixbuf);
+	g_object_unref(pixbuf);
+
+	g_signal_connect(G_OBJECT(indicator), INDICATE_INDICATOR_SIGNAL_DISPLAY,
+			G_CALLBACK(message_display), NULL);
+
+	indicate_indicator_show(indicator);
+
+	return indicator;
+}
+#endif // USE_INDICATE
+
+void messaging_menu_add_new_msg(FxMain *fxmain, Contact *cnt)
+{
+#ifdef USE_INDICATE
+	gchar *sid = 0;
+	gchar count[128];
+
+	sid = fetion_sip_get_sid_by_sipuri(cnt->sipuri);
+
+	IndicateIndicator *indicator = get_indicator(fxmain->indserver, sid);
+	
+	gint count_int = atoi(indicate_indicator_get_property(indicator, INDICATE_INDICATOR_MESSAGES_PROP_COUNT));
+	snprintf(count, sizeof(count) - 1, "%d", count_int + 1);
+	indicate_indicator_set_property(indicator, INDICATE_INDICATOR_MESSAGES_PROP_COUNT, count);
+#if 0
+	GTimeVal time;
+	g_get_current_time(&time);
+	indicate_indicator_set_property_time(ind, "time", &time);
+#endif
+	indicate_indicator_set_property(indicator, INDICATE_INDICATOR_MESSAGES_PROP_NAME,
+			*(cnt->localname) == '\0'?cnt->nickname: cnt->localname);
+	indicate_indicator_set_property(indicator, INDICATE_INDICATOR_MESSAGES_PROP_ATTENTION, "GET A MESSAGE");
+	g_free(sid);
+#endif // USE_INDICATE
+}
+
 void fx_main_process_message(FxMain* fxmain , FetionSip* sip , const gchar* sipmsg)
 {
 	Message   *msg;
@@ -620,12 +781,12 @@ void fx_main_process_message(FxMain* fxmain , FetionSip* sip , const gchar* sipm
 
 	if(!fxchat || fxchat->hasFocus == CHAT_DIALOG_NOT_FOCUSED){
 		/* chat window does not exist */
-		if(!fxchat){
+		if(!fxchat) {
 
 			/* auto popup enabled */
-			if(config->autoPopup == AUTO_POPUP_ENABLE){
+			if(config->autoPopup == AUTO_POPUP_ENABLE) {
 				fxchat = fx_main_create_chat_window(fxmain , msg->sipuri);
-				if(!fxchat){
+				if(!fxchat) {
 					fetion_message_free(msg);
 					gdk_threads_leave();
 					return;
@@ -635,41 +796,45 @@ void fx_main_process_message(FxMain* fxmain , FetionSip* sip , const gchar* sipm
 				fetion_message_free(msg);
 				gdk_threads_leave();
 				return;
-			}else{
+			} else {
 				/* chat window doesn`t exist and auto-pupup wasn`t enabled
 				 * just push message into message queue,wait for user action */
 				mitem = fx_list_new(msg);
 				fx_list_append(fxmain->mlist , mitem );
 				popup_msg_notify(fxmain, senderContact, msg);
+				messaging_menu_add_new_msg(fxmain, senderContact);
 			}
 		/* chat window exist,but not focused */
-		}else{
+		} else {
 			fx_chat_add_message(fxchat , msg->message,
 						 &(msg->sendtime) , 0 , msg->sysback);
+			messaging_menu_add_new_msg(fxmain, senderContact);
 			fxchat->unreadMsgCount ++;
 			fx_chat_update_window(fxchat);
 		}
 
-		sprintf(path , "%s/%s.jpg" , config->iconPath , sid);
-		g_free(sid);
-		pb = gdk_pixbuf_new_from_file(path , NULL);
-		if(!pb)
-			pb = gdk_pixbuf_new_from_file(SKIN_DIR"online.svg" , NULL);
-		gtk_status_icon_set_from_pixbuf(GTK_STATUS_ICON(fxmain->trayIcon) , pb);
-		g_object_unref(pb);
+		if(USE_STATUS_ICON(config)) { /* set the action of the status icon */
+			snprintf(path, sizeof(path) - 1, "%s/%s.jpg", config->iconPath, sid);
+			g_free(sid);
 
-		gtk_status_icon_set_blinking(GTK_STATUS_ICON(fxmain->trayIcon) , TRUE);
-		g_signal_handler_disconnect(fxmain->trayIcon , fxmain->iconConnectId);
+			pb = gdk_pixbuf_new_from_file(path, NULL);
+			if(!pb)	pb = gdk_pixbuf_new_from_file(SKIN_DIR"online.svg", NULL);
+			gtk_status_icon_set_from_pixbuf(GTK_STATUS_ICON(fxmain->trayIcon), pb);
+			g_object_unref(pb);
 
-		fxmain->iconConnectId = g_signal_connect(G_OBJECT(fxmain->trayIcon)
-							, "activate"
-							, GTK_SIGNAL_FUNC(fx_main_message_func)
-							, fxmain);
+			gtk_status_icon_set_blinking(GTK_STATUS_ICON(fxmain->trayIcon), TRUE);
+			g_signal_handler_disconnect(fxmain->trayIcon, fxmain->iconConnectId);
+
+			fxmain->iconConnectId = g_signal_connect(G_OBJECT(fxmain->trayIcon)
+								, "activate"
+								, GTK_SIGNAL_FUNC(fx_main_message_func)
+								, fxmain);
+		}
 
 		/* no message was pushed into the message queue,just free it */
 		if(fxchat && fxchat->hasFocus == CHAT_DIALOG_NOT_FOCUSED)
 			fetion_message_free(msg);
-	}else{
+	} else {
 		fx_chat_add_message(fxchat , msg->message,
 					   	&(msg->sendtime) , 0, msg->sysback);
 
@@ -680,9 +845,10 @@ void fx_main_process_message(FxMain* fxmain , FetionSip* sip , const gchar* sipm
 	if(config->isMute == MUTE_DISABLE)
 		fx_sound_play_file(RESOURCE_DIR"newmessage.wav");
 }
-void fx_main_process_user_left(FxMain* fxmain , const gchar* msg)
+
+void fx_main_process_user_left(FxMain *fxmain, const gchar *msg)
 {
-	gchar         *sipuri;
+	gchar        *sipuri;
 	FxList       *clist;
 	FxChat       *fxchat;
 	Conversation *conv;
@@ -804,6 +970,17 @@ static void process_share_action_cancel(FxMain *fxmain
 
 }
 #endif
+
+static void *input_func(void *data)
+{
+	FxChat *fxchat = (FxChat*)data;
+	gdk_threads_enter();
+	if(--fxchat->inputRefcnt <= 0)
+		fx_chat_clear_input(fxchat);
+	gdk_threads_leave();
+	return (void*)0;
+}
+
 void fx_main_process_incoming(FxMain* fxmain
 		, FetionSip* sip , const gchar* sipmsg)
 {
@@ -828,6 +1005,15 @@ void fx_main_process_incoming(FxMain* fxmain
 				debug_info("Received a nudge from %s" , sipuri);
 				break;
 			}
+		case INCOMING_INPUT:
+			{
+				if(!(fxchat = fx_list_find_chat_by_sipuri(fxmain->clist , sipuri))) return;
+				gdk_threads_enter();
+				fxchat->inputRefcnt ++;
+				fx_chat_set_input(fxchat);
+				g_timeout_add_seconds(3, (GSourceFunc)input_func, fxchat);
+				gdk_threads_leave();
+			}
 		case INCOMING_SHARE_CONTENT :
 			{
 				switch(action){
@@ -1045,18 +1231,21 @@ gboolean fx_main_delete(GtkWidget *widget , GdkEvent *UNUSED(event) , gpointer d
 	Config *config;
 
 
-	int     window_width;
-	int     window_height;
-	int     window_x;
-	int     window_y;
+	gint     window_width;
+	gint     window_height;
+	gint     window_x;
+	gint     window_y;
+	gint	 useMessagingMenu = 0;
+#ifdef USE_INDICATE
+	useMessagingMenu = 1;
+#endif
 
-	printf("fx_main_delete\n");
-
-	if(!fxmain->window)
-		return TRUE;
+	if(!fxmain->window)	return TRUE;
 
 	if(fxmain->user){
 		config = fxmain->user->config;
+		if(!USE_STATUS_ICON(config) && !useMessagingMenu) return TRUE;
+
 		gtk_window_get_position(GTK_WINDOW(fxmain->window),
 				&window_x, &window_y);
 		config->window_pos_x = window_x;
@@ -1297,7 +1486,7 @@ void fx_main_tray_popmenu_func(
 			args->fxmain = fxmain;
 			args->type = presence[i].type;
 
-			sprintf(stateMenu , "%s      " , presence[i].name);
+			sprintf(stateMenu , "%s      " , _(presence[i].name));
 			fx_main_create_menu(stateMenu , presence[i].icon
 							 , submenu , fx_main_set_state_clicked , args);
 		}
@@ -1346,7 +1535,8 @@ int main(int argc , char* argv[])
 		return fx_cli_exec();
 	}
 
-	FxMain* fxmain = fx_main_new();
+	/* use the GLOBAL fxmain variable */
+	fxmain = fx_main_new();
 	
 	if(!g_thread_supported())
 		g_thread_init(NULL);
@@ -1531,6 +1721,23 @@ void fx_main_message_func(GtkWidget *UNUSED(widget) , gpointer data)
 	notify_notification_close(fxmain->notify , NULL);
 #endif
 
+#ifdef USE_INDICATE
+	GSList *iterator = NULL;
+	IndicateIndicator *indicator = NULL;
+
+	for (iterator = indicators; iterator; iterator = iterator->next) {
+		indicator = (IndicateIndicator*)iterator->data;
+		/* clear attention */
+		indicate_indicator_set_property(indicator, INDICATE_INDICATOR_MESSAGES_PROP_ATTENTION, "");
+		/* remove the indicators from the indicate server */
+		indicate_server_remove_indicator(fxmain->indserver, indicator);
+	}
+
+	/* free the memory allocated for the indicator list */
+	g_slist_free(indicators);
+	indicators = NULL;
+#endif
+
 	while(cur != fxmain->mlist){
 		msg = (Message*)(cur->data);
 
@@ -1564,15 +1771,18 @@ void fx_main_message_func(GtkWidget *UNUSED(widget) , gpointer data)
 		free(tmp);
 	}
 	fx_head_set_state_image(fxmain , fxmain->user->state);
-	gtk_status_icon_set_blinking(
-			GTK_STATUS_ICON(fxmain->trayIcon) , FALSE);
-	g_signal_handler_disconnect(fxmain->trayIcon,
-			fxmain->iconConnectId);
-	fxmain->iconConnectId = g_signal_connect(
-						G_OBJECT(fxmain->trayIcon)
-						 , "activate"
-						 , GTK_SIGNAL_FUNC(fx_main_tray_activate_func)
-						 , fxmain);
+	
+	if(USE_STATUS_ICON(fxmain->user->config)) { /* set the action of the status icon */
+		gtk_status_icon_set_blinking(
+				GTK_STATUS_ICON(fxmain->trayIcon) , FALSE);
+		g_signal_handler_disconnect(fxmain->trayIcon,
+				fxmain->iconConnectId);
+		fxmain->iconConnectId = g_signal_connect(
+							G_OBJECT(fxmain->trayIcon)
+							 , "activate"
+							 , GTK_SIGNAL_FUNC(fx_main_tray_activate_func)
+							 , fxmain);
+	}
 
 	foreach_list(fxmain->clist, cur){
 		fxchat = (FxChat*)cur->data;
@@ -1652,9 +1862,6 @@ void fx_main_system_setting_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 	FxSet *fxset = fx_set_new(fxmain);
 	fx_set_initialize(fxset);
 	g_object_set(fxset->notebook , "page" , 1 , NULL);
-	gtk_window_set_position(GTK_WINDOW(fxset->dialog) , GTK_WIN_POS_CENTER);
-	gtk_dialog_run(GTK_DIALOG(fxset->dialog));
-	gtk_widget_destroy(fxset->dialog);
 }
 
 void fx_main_set_state_clicked(GtkWidget *widget , gpointer data)
