@@ -15,7 +15,18 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.            *
+ *                                                                         *
+ *   OpenSSL linking exception                                             *
+ *   --------------------------                                            *
+ *   If you modify this Program, or any covered work, by linking or        *
+ *   combining it with the OpenSSL project's "OpenSSL" library (or a       *
+ *   modified version of that library), containing parts covered by        *
+ *   the terms of OpenSSL/SSLeay license, the licensors of this            *
+ *   Program grant you additional permission to convey the resulting       *
+ *   work. Corresponding Source for a non-source form of such a            *
+ *   combination shall include the source code for the parts of the        *
+ *   OpenSSL library used as well as that of the covered work.             *
  ***************************************************************************/
 
 #include "fx_include.h"
@@ -92,6 +103,11 @@ static void fx_set_on_ok_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 		else
 			config->closeFetionShow = CLOSE_FETION_SHOW_DISABLE;
 
+		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fxset->useStatusIcon)))
+			config->useStatusIcon = USE_STATUS_ICON_DISABLE;
+		else
+			config->useStatusIcon = USE_STATUS_ICON_ENABLE;
+
 		gtk_text_buffer_get_start_iter(buffer , &startIter);
 		gtk_text_buffer_get_end_iter(buffer , &endIter);
 		autoReplyMsg = gtk_text_buffer_get_text(buffer , &startIter , &endIter , TRUE);
@@ -100,6 +116,7 @@ static void fx_set_on_ok_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 
 		if(autoReplyMsg != NULL)
 			strcpy(config->autoReplyMessage , autoReplyMsg);
+		fetion_config_set_use_status_icon(config);
 		fetion_config_save(user);
 	}
 	else
@@ -138,12 +155,12 @@ static void fx_set_on_ok_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 				? _("Click here to input signature") : user->impression);
 				}
 	}
-	gtk_dialog_response(GTK_DIALOG(fxset->dialog) , GTK_RESPONSE_CANCEL);
+	gtk_widget_destroy(fxset->dialog);
 }
 
 static void fx_set_on_cancel_clicked(GtkWidget *UNUSED(widget) , gpointer data)
 {
-	gtk_dialog_response(GTK_DIALOG(data) , GTK_RESPONSE_CANCEL);
+	gtk_widget_destroy(GTK_WIDGET(data));
 }
 
 static void fx_set_on_autoreply_toggled(GtkWidget *widget , gpointer data)
@@ -172,15 +189,16 @@ void fx_set_initialize(FxSet* fxset)
 
 	GtkWidget *ok_button = NULL;
 	GtkWidget* cancel_button = NULL;
+	GtkBox *vbox , *action_area;
 
 	GdkPixbuf* pb = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"online.svg",
 				   22, 22, NULL);
 
-	fxset->dialog = gtk_dialog_new();
+	fxset->dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_position(GTK_WINDOW(fxset->dialog), GTK_WIN_POS_CENTER);
 	gtk_window_set_icon(GTK_WINDOW(fxset->dialog) , pb);
 	g_object_unref(pb);
-	gtk_dialog_set_has_separator(GTK_DIALOG(fxset->dialog)
-							   , FALSE);
+
 	gtk_widget_set_usize(fxset->dialog , 500 , 360);
 	gtk_window_set_title(GTK_WINDOW(fxset->dialog) , _("OpenFetion Preference"));
 
@@ -188,8 +206,11 @@ void fx_set_initialize(FxSet* fxset)
 	gtk_widget_set_usize(fxset->notebook , 490 , 320);
 	gtk_notebook_set_show_border(GTK_NOTEBOOK(fxset->notebook) , FALSE);
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(fxset->notebook) , GTK_POS_TOP);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(fxset->dialog)->vbox)
-					 , fxset->notebook , FALSE , FALSE , 0);
+
+	vbox = GTK_BOX(gtk_vbox_new (FALSE, 0));
+	gtk_container_add(GTK_CONTAINER(fxset->dialog), GTK_WIDGET(vbox));
+
+	gtk_box_pack_start(vbox, fxset->notebook , FALSE , FALSE , 0);
 
 	fxset->psetting = gtk_vbox_new(FALSE , FALSE);
 	psetting_label = gtk_label_new(_("Personal Settings"));
@@ -203,15 +224,19 @@ void fx_set_initialize(FxSet* fxset)
 						   , fxset->ssetting
 						   , ssetting_label);
 
-
-	ok_button = gtk_button_new_with_label(_("OK"));
-	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(fxset->dialog)->action_area) , ok_button);
-	g_signal_connect(ok_button , "clicked" , G_CALLBACK(fx_set_on_ok_clicked) , fxset);
+	action_area = GTK_BOX(gtk_hbox_new(FALSE , 0));
+	gtk_box_pack_start(GTK_BOX(vbox) , GTK_WIDGET(action_area) , FALSE , FALSE , 5);
 
 	cancel_button = gtk_button_new_with_label(_("Cancel"));
-	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(fxset->dialog)->action_area) , cancel_button);
+	gtk_widget_set_usize(cancel_button, 100, 30);
+	gtk_box_pack_end(action_area , cancel_button, FALSE, FALSE, 5);
 	g_signal_connect(cancel_button , "clicked" , G_CALLBACK(fx_set_on_cancel_clicked) , fxset->dialog);
 
+	ok_button = gtk_button_new_with_label(_("OK"));
+	gtk_widget_set_usize(ok_button, 100, 30);
+	gtk_box_pack_end(action_area , ok_button, FALSE, FALSE, 5);
+	g_signal_connect(ok_button , "clicked" , G_CALLBACK(fx_set_on_ok_clicked) , fxset);
+
 	fx_set_initialize_personal(fxset);
 	fx_set_initialize_system(fxset);
 
@@ -276,18 +301,20 @@ void fx_set_bind_system(FxSet* fxset)
 	else
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxset->closeShowBtn), FALSE);
 
+	if(config->useStatusIcon == USE_STATUS_ICON_ENABLE)
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxset->useStatusIcon), FALSE);
+	else
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxset->useStatusIcon), TRUE);
+
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(fxset->apEty));
 	gtk_text_buffer_get_start_iter(buffer , &startIter);
 	gtk_text_buffer_get_end_iter(buffer , &endIter);
 	gtk_text_buffer_delete(buffer , &startIter , &endIter);
 
-	if(strlen(config->autoReplyMessage) == 0)
-	{
+	if(strlen(config->autoReplyMessage) == 0) {
 		autoReplyMsg = _("Sorry, I am not in now, and will reply to you soon");
 		gtk_text_buffer_insert(buffer , &startIter , autoReplyMsg , strlen(autoReplyMsg));
-	}
-	else
-	{
+	} else {
 		gtk_text_buffer_insert(buffer , &startIter , config->autoReplyMessage , strlen(config->autoReplyMessage));
 	}
 }
@@ -484,6 +511,9 @@ void fx_set_initialize_system(FxSet* fxset)
 	fxset->closeShowBtn = gtk_check_button_new_with_label(_("Disable Fetion Show"));
 	gtk_fixed_put(GTK_FIXED(fixed), fxset->closeShowBtn, 40, 132);
 
+	fxset->useStatusIcon = gtk_check_button_new_with_label(_("Don't Use Status Icon(restart)"));
+	gtk_fixed_put(GTK_FIXED(fixed), fxset->useStatusIcon, 230, 132);
+
 	label2 = gtk_label_new("");
 	gtk_label_set_markup(GTK_LABEL(label2) , _("<b>Auto Reply</b>"));
 	gtk_fixed_put(GTK_FIXED(fixed) , label2 , 20 , 162);
